@@ -25,6 +25,7 @@ intended to be used by the backends themselves.
 """
 
 import os
+import sys
 import socket
 import time
 import re
@@ -44,12 +45,50 @@ from duplicity.errors import ConflictingScheme
 from duplicity.errors import InvalidBackendURL
 from duplicity.errors import UnsupportedBackendScheme
 
+import duplicity.backends
+
+
 # todo: this should really NOT be done here
 socket.setdefaulttimeout(globals.timeout)
 
+_forced_backend = None
 _backends = {}
 
 
+def import_backends():
+    """
+    Import files in the duplicity/backends directory where
+    the filename ends in 'backend.py' and ignore the rest.
+
+    @rtype: void
+    @return: void
+    """
+    path = duplicity.backends.__path__[0]
+    assert path.endswith("duplicity/backends"), duplicity.backends.__path__
+
+    files = os.listdir(path)
+    for fn in files:
+        if fn.endswith("backend.py"):
+            fn = fn[:-3]
+            imp = "duplicity.backends.%s" % (fn,)
+            try:
+                __import__(imp)
+                res = "Succeeded"
+            except:
+                res = "Failed: " + str(sys.exc_info()[1])
+            log.Info("Import of %s %s" % (imp, res))
+        else:
+            continue
+
+
+def force_backend(backend):
+    """
+    Forces the use of a particular backend, regardless of schema
+    """
+    global _forced_backend
+    _forced_backend = backend
+
+
 def register_backend(scheme, backend_factory):
     """
     Register a given backend factory responsible for URL:s with the
@@ -69,13 +108,26 @@ def register_backend(scheme, backend_factory):
     assert callable(backend_factory), "backend factory must be callable"
 
     if scheme in _backends:
-        raise ConflictingSchemeError("the scheme %s already has a backend "
-                                     "associated with it"
-                                     "" % (scheme,))
+        raise ConflictingScheme("the scheme %s already has a backend "
+                                "associated with it"
+                                "" % (scheme,))
 
     _backends[scheme] = backend_factory
 
 
+def is_backend_url(url_string):
+    """
+    @return Whether the given string looks like a backend URL.
+    """
+    pu = ParsedUrl(url_string)
+
+    # Be verbose to actually return True/False rather than string.
+    if pu.scheme:
+        return True
+    else:
+        return False
+
+
 def get_backend(url_string):
     """
     Instantiate a backend suitable for the given URL, or return None
@@ -83,15 +135,19 @@ def get_backend(url_string):
 
     Raise InvalidBackendURL if the URL is not a valid URL.
     """
+    if not is_backend_url(url_string):
+        return None
+
     pu = ParsedUrl(url_string)
 
     # Implicit local path
-    if not pu.scheme:
-        return None
+    assert pu.scheme, "should be a backend url according to is_backend_url"
 
-    global _backends
+    global _backends, _forced_backend
 
-    if not pu.scheme in _backends:
+    if _forced_backend:
+        return _forced_backend(pu)
+    elif not pu.scheme in _backends:
         raise UnsupportedBackendScheme(url_string)
     else:
         return _backends[pu.scheme](pu)
@@ -187,19 +243,24 @@ class ParsedUrl:
             self.password = pu.password
         except:
             raise InvalidBackendURL("Syntax error (password) in: %s" % url_string)
+        if self.password:
+            self.password = urllib.unquote(self.password)
+        else:
+            self.password = None
 
         try:
             self.hostname = pu.hostname
         except:
             raise InvalidBackendURL("Syntax error (hostname) in: %s" % url_string)
 
-        if self.scheme not in ['rsync']:
-            try:
-                self.port = pu.port
-            except:
-                raise InvalidBackendURL("Syntax error (port) in: %s" % url_string)
-        else:
-            self.port = None
+        # init to None, overwrite with actual value on success
+        self.port = None
+        try:
+            self.port = pu.port
+        except:
+            # old style rsync://host::[/]dest, are still valid, though they contain no port
+            if not ( self.scheme in ['rsync'] and re.search('::[^:]*$', self.url_string)):
+                raise InvalidBackendURL("Syntax error (port) in: %s A%s B%s C%s" % (url_string, (self.scheme in ['rsync']), re.search('::[^:]+$', self.netloc), self.netloc ) )
 
         # This happens for implicit local paths.
         if not pu.scheme:
@@ -272,6 +333,9 @@ class Backend:
         """
         raise NotImplementedError()
 
+    """ use getpass by default, inherited backends may overwrite this behaviour """
+    use_getpass = True
+
     def get_password(self):
         """
         Return a password for authentication purposes. The password
@@ -285,8 +349,12 @@ class Backend:
         try:
             password = os.environ['FTP_PASSWORD']
         except KeyError:
-            password = getpass.getpass("Password for '%s': " % self.parsed_url.hostname)
-            os.environ['FTP_PASSWORD'] = password
+            if self.use_getpass:
+                password = getpass.getpass("Password for '%s@%s': " %
+                                           (self.parsed_url.username,self.parsed_url.hostname) )
+                os.environ['FTP_PASSWORD'] = password
+            else:
+                password = None
         return password
 
     def munge_password(self, commandline):
@@ -304,76 +372,78 @@ class Backend:
         else:
             return commandline
 
+    """
+    DEPRECATED:
+    run_command(_persist) - legacy wrappers for subprocess_popen(_persist)
+    """
     def run_command(self, commandline):
-        """
-        Execute the given command line, interpreted as a shell
-        command, with logging and error detection. If execution fails,
-        raise a BackendException.
-        """
-        private = self.munge_password(commandline)
-        log.Info(_("Running '%s'") % private)
-        if os.system(commandline):
-            raise BackendException("Error running '%s'" % private)
-
+        return self.subprocess_popen(commandline)
     def run_command_persist(self, commandline):
-        """
-        Like run_command(), but repeat the attempt several times (with
-        a delay in between) if it fails.
-        """
-        private = self.munge_password(commandline)
-        for n in range(1, globals.num_retries+1):
-            if n > 1:
-                # sleep before retry
-                time.sleep(30)
-            log.Info(gettext.ngettext("Running '%s' (attempt #%d)",
-                                      "Running '%s' (attempt #%d)", n) %
-                                      (private, n))
-            if not os.system(commandline):
-                return
-            log.Warn(gettext.ngettext("Running '%s' failed (attempt #%d)",
-                                      "Running '%s' failed (attempt #%d)", n) %
-                                      (private, n), 1)
-        log.Warn(gettext.ngettext("Giving up trying to execute '%s' after %d attempt",
-                                 "Giving up trying to execute '%s' after %d attempts",
-                                 globals.num_retries) % (private, globals.num_retries))
-        raise BackendException("Error running '%s'" % private)
+        return self.subprocess_popen_persist(commandline)
 
+    """
+    DEPRECATED:
+    popen(_persist) - legacy wrappers for subprocess_popen(_persist)
+    """
     def popen(self, commandline):
+        result, stdout, stderr = self.subprocess_popen(commandline)
+        return stdout
+    def popen_persist(self, commandline):
+        result, stdout, stderr = self.subprocess_popen_persist(commandline)
+        return stdout
+
+    def _subprocess_popen(self, commandline):
         """
-        Like run_command(), but capture stdout and return it (the
-        contents read from stdout) as a string.
+        For internal use.
+        Execute the given command line, interpreted as a shell command.
+        Returns int Exitcode, string StdOut, string StdErr
+        """
+        from subprocess import Popen, PIPE
+        p = Popen(commandline, shell=True, stdout=PIPE, stderr=PIPE)
+        stdout, stderr = p.communicate()
+
+        return p.returncode, stdout, stderr
+
+    def subprocess_popen(self, commandline):
+        """
+        Execute the given command line with error check.
+        Returns int Exitcode, string StdOut, string StdErr
+
+        Raise a BackendException on failure.
         """
         private = self.munge_password(commandline)
         log.Info(_("Reading results of '%s'") % private)
-        fout = os.popen(commandline)
-        results = fout.read()
-        if fout.close():
+        result, stdout, stderr = self._subprocess_popen(commandline)
+        if result != 0:
             raise BackendException("Error running '%s'" % private)
-        return results
+        return result, stdout, stderr
 
-    def popen_persist(self, commandline):
+    def subprocess_popen_persist(self, commandline):
         """
-        Like run_command_persist(), but capture stdout and return it
-        (the contents read from stdout) as a string.
+        Execute the given command line with error check.
+        Retries globals.num_retries times with 30s delay.
+        Returns int Exitcode, string StdOut, string StdErr
+
+        Raise a BackendException on failure.
         """
         private = self.munge_password(commandline)
         for n in range(1, globals.num_retries+1):
+            # sleep before retry
             if n > 1:
-                # sleep before retry
                 time.sleep(30)
             log.Info(_("Reading results of '%s'") % private)
-            fout = os.popen(commandline)
-            results = fout.read()
-            result_status = fout.close()
-            if not result_status:
-                return results
-            elif result_status == 1280 and self.parsed_url.scheme == 'ftp':
+            result, stdout, stderr = self._subprocess_popen(commandline)
+            if result == 0:
+                return result, stdout, stderr
+            elif result == 1280 and self.parsed_url.scheme == 'ftp':
                 # This squelches the "file not found" result fromm ncftpls when
                 # the ftp backend looks for a collection that does not exist.
                 return ''
             log.Warn(gettext.ngettext("Running '%s' failed (attempt #%d)",
                                      "Running '%s' failed (attempt #%d)", n) %
                                       (private, n))
+            if stdout or stderr:
+                log.Warn(_("Error is:\n%s") % stderr + (stderr and stdout and "\n") + stdout)
         log.Warn(gettext.ngettext("Giving up trying to execute '%s' after %d attempt",
                                   "Giving up trying to execute '%s' after %d attempts",
                                   globals.num_retries) % (private, globals.num_retries))
